'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.demux = exports.node = undefined;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = combiner;
var node = exports.node = initNode(defaultCombiner);

var demux = exports.demux = initDemux(defaultCombiner);

var INITIAL_ACTION = '@@redux-combiner/INIT';

function defaultCombiner(reducersMap) {

    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var action = arguments[1];


        var result = isArray(state) ? [] : {};

        var changed = false;

        for (var k in reducersMap) {
            if (hasOwn(reducersMap, k)) {

                var reducer = reducersMap[k];

                var prevState = state[k];
                var nextState = reducer(prevState, action);

                if (typeof nextState === 'undefined') {
                    throw new Error('undefined state returned for key "' + k + '" on "' + action.type + '" action.');
                }

                result[k] = nextState;
                changed = changed || prevState !== nextState;
            }
        }

        return changed ? result : state;
    };
}

function combiner(combineReducers) {

    return {
        node: initNode(combineReducers),
        demux: initDemux(combineReducers)
    };
}

function initNode(combineReducers) {

    return function (initial) {

        checkInitial(initial, node);

        var innerReducer = getInnerReducer(initial, combineReducers);

        var fn = function fn() {
            var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getDefault(initial);
            var action = arguments[1];


            state = applyOwnReducers(state, action, fn.reducers);
            state = innerReducer(state, action);

            return state;
        };

        defineActions(fn, innerReducer.actions);
        Object.defineProperty(fn, 'reducers', { value: new _map2.default() });
        Object.defineProperty(fn, 'on', { value: addActionsReducers });

        return fn;
    };
}

function initDemux(combineReducers) {

    return function (initial, schema, options) {

        checkInitial(initial, demux);

        var getChildKey = createActionKeyGetter(options);

        var childReducer = getChildReducer(schema, getChildKey, combineReducers);

        var inited = false;

        var fn = function fn() {
            var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getDefault(initial);
            var action = arguments[1];


            state = applyOwnReducers(state, action, fn.reducers);

            if (inited) {
                state = childReducer(state, action);
            }

            inited = true;

            return state;
        };

        defineActions(fn, childReducer.actions);
        Object.defineProperty(fn, 'reducers', { value: new _map2.default() });
        Object.defineProperty(fn, 'on', { value: addActionsReducers });

        return fn;
    };
}

function checkInitial(initial, frame) {
    if (typeof initial === 'undefined') {
        var e = new Error('node must be initialized.');
        Error.captureStackTrace(e, frame);
        throw e;
    }
}

function getDefault(initial) {

    if (isFunction(initial)) {
        return initial(undefined, { type: INITIAL_ACTION });
    }

    if (isObject(initial)) {
        for (var k in initial) {
            if (hasOwn(initial, k)) {
                initial[k] = getDefault(initial[k]);
            }
        }
    }

    return initial;
}

function identity(data) {
    return data;
}

function getInnerReducer(initial, combineReducers, noWrap) {

    if (isFunction(initial)) {
        if (!hasOwn(initial, 'actions')) {
            defineActions(initial, null);
        }
        return initial;
    }

    if (!isObject(initial)) {
        return noWrap ? initial : identity;
    }

    var reducers = {};
    var reducersActions = [];
    var hasFn = false;

    for (var k in initial) {
        if (hasOwn(initial, k)) {

            var innerReducer = getInnerReducer(initial[k], combineReducers, true);

            if (isFunction(innerReducer)) {
                hasFn = true;
                reducers[k] = innerReducer;
                reducersActions = reducersActions === null ? null : !innerReducer.actions ? null : [].concat((0, _toConsumableArray3.default)(reducersActions), (0, _toConsumableArray3.default)(innerReducer.actions));
            }
        }
    }

    if (hasFn) {

        var fn = function fn(state, action) {

            var stateReducers = {};

            var _loop = function _loop(_k) {
                if (hasOwn(state, _k)) {
                    if (reducers[_k] && (!reducers[_k].actions || reducers[_k].actions.has(action.type))) {
                        stateReducers[_k] = reducers[_k];
                    } else {
                        stateReducers[_k] = function () {
                            return state[_k];
                        };
                    }
                }
            };

            for (var _k in state) {
                _loop(_k);
            }

            var reducer = combineReducers(stateReducers);

            return reducer(state, action);
        };

        defineActions(fn, reducersActions);

        return fn;
    } else {
        return noWrap ? initial : identity;
    }
}

function getChildReducer(schema, getChildKey, combineReducers) {

    var schemaReducer = getInnerReducer(schema, combineReducers);

    var fn = function fn(state, action) {

        if (!isObject(state)) {
            return state;
        }

        var childKey = getChildKey(state, action);

        if (!hasOwn(state, childKey)) {
            return state;
        }

        var reducers = {};

        var _loop2 = function _loop2(k) {
            if (hasOwn(state, k)) {
                reducers[k] = function () {
                    return state[k];
                };
            }
        };

        for (var k in state) {
            _loop2(k);
        }

        reducers[childKey] = schemaReducer;

        var reducer = combineReducers(reducers);

        return reducer(state, action);
    };

    defineActions(fn, schemaReducer.actions);

    return fn;
}

function applyOwnReducers(state, action, actionReducers) {

    var reducers = actionReducers.get(action.type);

    if (reducers) {
        state = reducers.reduce(function (state, reducer) {
            return reducer(state, action);
        }, state);
    }

    return state;
}

function createActionKeyGetter() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var actionKey = options.actionKey,
        getKey = options.getKey;


    if (isFunction(options)) {
        getKey = options;
    } else if (isString(options) || isArray(options)) {
        actionKey = options;
    }

    if (isFunction(getKey)) {
        return getKey;
    }

    if (!actionKey) {
        return getDefaultActionKey;
    }

    if (!isArray(actionKey)) {
        if (isString(actionKey)) {
            actionKey = actionKey.split('.');
        } else {
            actionKey = [actionKey];
        }
    }

    return function (state, action) {
        return getValueByPath(action, actionKey);
    };
}

function getDefaultActionKey(state, action) {
    return isArray(state) ? action.index : action.id;
}

function getValueByPath(obj, path) {

    var pointer = obj;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(path), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var p = _step.value;


            if (!hasOwn(pointer, p)) {
                return;
            }

            pointer = pointer[p];
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return pointer;
}

function addActionsReducers(actionTypes, reducers) {

    if (!isArray(actionTypes)) {
        actionTypes = [actionTypes];
    }

    reducers = getReducers(reducers);

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(actionTypes), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var actionType = _step2.value;
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {

                for (var _iterator3 = (0, _getIterator3.default)(reducers), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var reducer = _step3.value;


                    var actionReducers = this.reducers.get(actionType);

                    if (!actionReducers) {
                        actionReducers = [];
                        this.actions && this.actions.add(actionType);
                        this.reducers.set(actionType, actionReducers);
                    }

                    actionReducers.push(reducer);
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return this;
}

function getReducers(reducers) {

    if (isFunction(reducers)) {
        return [reducers];
    }

    if (!isArray(reducers)) {
        return [function () {
            return reducers;
        }];
    }

    var containsReducer = false;

    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
        for (var _iterator4 = (0, _getIterator3.default)(reducers), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var reducer = _step4.value;

            if (isFunction(reducer)) {
                containsReducer = true;
                break;
            }
        }
    } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
            }
        } finally {
            if (_didIteratorError4) {
                throw _iteratorError4;
            }
        }
    }

    if (containsReducer) {
        return reducers.map(function (reducer) {
            return isFunction(reducer) ? reducer : function () {
                return reducer;
            };
        });
    }

    return [function () {
        return reducers;
    }];
}

function defineActions(fn, actions) {
    Object.defineProperty(fn, 'actions', {
        value: actions === null ? null : new _set2.default(actions)
    });
}

function isObject(obj) {
    return obj && (typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) === 'object';
}

function isArray(arr) {
    return Array.isArray(arr);
}

function isFunction(fn) {
    return typeof fn === 'function';
}

function isString(str) {
    return typeof str === 'string';
}

function hasOwn(obj, prop) {
    return Object.prototype.hasOwnProperty.call(obj, prop);
}